@@ -1,15 +1,15 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
+require_relative "spec_helper"
 
 describe "A new Database" do
   before do
     @db = Sequel::Database.new(1 => 2, :logger => 3)
   end
-  after do
-    Sequel.quote_identifiers = false
-    Sequel.identifier_input_method = nil
-    Sequel.identifier_output_method = nil
-  end
   
+  it "should not allow dup/clone" do
+    proc{@db.dup}.must_raise NoMethodError
+    proc{@db.clone}.must_raise NoMethodError
+  end
+
   it "should receive options" do
     @db.opts[1].must_equal 2
     @db.opts[:logger].must_equal 3  
@@ -26,6 +26,7 @@ describe "A new Database" do
   it "should support :preconnect option to preconnect to database" do
     @db.pool.size.must_equal 0
     c = Class.new(Sequel::Database) do
+      def dataset_class_default; Sequel::Dataset end
       def connect(_)
         :connect
       end
@@ -59,7 +60,7 @@ describe "A new Database" do
   it "should have the connection pool use the connect method to get connections" do
     cc = nil
     d = Sequel::Database.new
-    meta_def(d, :connect){|c| 1234}
+    d.define_singleton_method(:connect){|c| 1234}
     d.synchronize {|c| cc = c}
     cc.must_equal 1234
   end
@@ -80,21 +81,13 @@ describe "A new Database" do
   end
 
   it "should just use a :uri option for jdbc with the full connection string" do
-    db = Sequel::Database.stub(:adapter_class, Sequel::Database) do
+    db = Sequel::Database.stub(:adapter_class, Class.new(Sequel::Database){def connect(*); Object.new end}) do
       Sequel.connect('jdbc:test://host/db_name')
     end
     db.must_be_kind_of(Sequel::Database)
     db.opts[:uri].must_equal 'jdbc:test://host/db_name'
   end
 
-  it "should just use a :uri option for do with the full connection string" do
-    db = Sequel::Database.stub(:adapter_class, Sequel::Database) do
-      Sequel.connect('do:test://host/db_name')
-    end
-    db.must_be_kind_of(Sequel::Database)
-    db.opts[:uri].must_equal 'do:test://host/db_name'
-  end
-
   it "should populate :adapter option when using connection string" do
     Sequel.connect('mock:/').opts[:adapter].must_equal "mock"
   end
@@ -106,7 +99,7 @@ describe "A new Database" do
 
   it 'should strip square brackets for ipv6 hosts' do
     Sequel.connect('mock://[::1]').opts[:host].must_equal "::1"
-  end if RUBY_VERSION >= '1.9.3'
+  end
 end
 
 describe "Database#freeze" do
@@ -169,84 +162,12 @@ describe "Database#log_info" do
   end
 end
 
-describe "Database#log_yield" do
-  before do
-    @o = Object.new
-    def @o.logs; @logs || []; end
-    def @o.warn(*args); (@logs ||= []) << [:warn] + args; end
-    def @o.method_missing(*args); (@logs ||= []) << args; end
-    def @o.to_ary; [self]; end
-    @db = Sequel::Database.new(:logger=>@o)
-  end
-
-  it "should yield to the passed block" do
-    a = nil
-    @db.log_yield('blah'){a = 1}
-    a.must_equal 1
-  end
-
-  it "should raise an exception if a block is not passed" do
-    proc{@db.log_yield('blah')}.must_raise LocalJumpError
-  end
-
-  it "should log message with duration at info level to all loggers" do
-    @db.log_yield('blah'){}
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :info
-    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
-  end
-
-  it "should respect sql_log_level setting" do
-    @db.sql_log_level = :debug
-    @db.log_yield('blah'){}
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :debug
-    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
-  end
-
-  it "should log message with duration at warn level if duration greater than log_warn_duration" do
-    @db.log_warn_duration = 0
-    @db.log_yield('blah'){}
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :warn
-    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
-  end
-
-  it "should log message with duration at info level if duration less than log_warn_duration" do
-    @db.log_warn_duration = 1000
-    @db.log_yield('blah'){}
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :info
-    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
-  end
-
-  it "should log message at error level if block raises an error" do
-    @db.log_warn_duration = 0
-    proc{@db.log_yield('blah'){raise Sequel::Error, 'adsf'}}.must_raise Sequel::Error
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :error
-    @o.logs.first.last.must_match(/\ASequel::Error: adsf: blah\z/)
-  end
-
-  it "should include args with message if args passed" do
-    @db.log_yield('blah', [1, 2]){}
-    @o.logs.length.must_equal 1
-    @o.logs.first.length.must_equal 2
-    @o.logs.first.first.must_equal :info
-    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah; \[1, 2\]\z/)
-  end
-end
-
 describe "Database#log_connection_yield" do
   before do
     @o = Object.new
     def @o.logs; @logs || []; end
     def @o.to_ary; [self]; end
+    def @o.warn(*args); (@logs ||= []) << [:warn] + args; end
     def @o.method_missing(*args); (@logs ||= []) << args; end
     @conn = Object.new
     @db = Sequel::Database.new(:logger=>@o)
@@ -269,11 +190,75 @@ describe "Database#log_connection_yield" do
     @o.logs.first.first.must_equal :info
     @o.logs.first.last.must_match(/\(conn: -?\d+\) some SQL\z/)
   end
+
+  it "should yield to the passed block" do
+    a = nil
+    @db.log_connection_yield('blah', @conn){a = 1}
+    a.must_equal 1
+  end
+
+  it "should raise an exception if a block is not passed" do
+    proc{@db.log_connection_yield('blah', @conn)}.must_raise LocalJumpError
+  end
+
+  it "should log message with duration at info level to all loggers" do
+    @db.log_connection_yield('blah', @conn){}
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :info
+    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
+  end
+
+  it "should respect sql_log_level setting" do
+    @db.sql_log_level = :debug
+    @db.log_connection_yield('blah', @conn){}
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :debug
+    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
+  end
+
+  it "should log message with duration at warn level if duration greater than log_warn_duration" do
+    @db.log_warn_duration = 0
+    @db.log_connection_yield('blah', @conn){}
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :warn
+    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
+  end
+
+  it "should log message with duration at info level if duration less than log_warn_duration" do
+    @db.log_warn_duration = 1000
+    @db.log_connection_yield('blah', @conn){}
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :info
+    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah\z/)
+  end
+
+  it "should log message at error level if block raises an error" do
+    @db.log_warn_duration = 0
+    proc{@db.log_connection_yield('blah', @conn){raise Sequel::Error, 'adsf'}}.must_raise Sequel::Error
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :error
+    @o.logs.first.last.must_match(/\ASequel::Error: adsf: blah\z/)
+  end
+
+  it "should include args with message if args passed" do
+    @db.log_connection_yield('blah', @conn, [1, 2]){}
+    @o.logs.length.must_equal 1
+    @o.logs.first.length.must_equal 2
+    @o.logs.first.first.must_equal :info
+    @o.logs.first.last.must_match(/\A\(\d\.\d{6}s\) blah; \[1, 2\]\z/)
+  end
 end
 
 describe "Database#uri" do
   before do
     @c = Class.new(Sequel::Database) do
+      def dataset_class_default; Sequel::Dataset end
+      def connect(*); Object.new end
       set_adapter_scheme :mau
     end
     
@@ -298,6 +283,7 @@ describe "Database.adapter_scheme and #adapter_scheme" do
     Sequel::Database.adapter_scheme.must_be_nil
 
     @c = Class.new(Sequel::Database) do
+      def dataset_class_default; Sequel::Dataset end
       set_adapter_scheme :mau
     end
     
@@ -328,10 +314,8 @@ describe "Database#dataset" do
     e.sql.must_equal 'SELECT * FROM miu'
   end
   
-  it "should provide a filtered #from dataset if a block is given" do
-    d = @db.from(:mau){x.sql_number > 100}
-    d.must_be_kind_of(Sequel::Dataset)
-    d.sql.must_equal 'SELECT * FROM mau WHERE (x > 100)'
+  it "should provide a #from dataset that supports virtual row blocks" do
+    @db.from{a(b)}.sql.must_equal 'SELECT * FROM a(b)'
   end
   
   it "should provide a #select dataset" do
@@ -433,6 +417,130 @@ describe "Database#extend_datasets" do
   end
 end
   
+describe "Database#extend_datasets custom methods" do
+  before do
+    @db = Sequel.mock
+  end
+
+  def ds
+    @db[:items]
+  end
+  
+  it "should have dataset_module support a where method" do
+    @db.extend_datasets{where :released, :released}
+    ds.released.sql.must_equal 'SELECT * FROM items WHERE released'
+    ds.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE (foo AND released)'
+  end
+
+  it "should have dataset_module support a having method" do
+    @db.extend_datasets{having(:released){released}}
+    ds.released.sql.must_equal 'SELECT * FROM items HAVING released'
+    ds.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE foo HAVING released'
+  end
+
+  it "should have dataset_module support an exclude method" do
+    @db.extend_datasets{exclude :released, :released}
+    ds.released.sql.must_equal 'SELECT * FROM items WHERE NOT released'
+    ds.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE (foo AND NOT released)'
+  end
+
+  it "should have dataset_module support an exclude_having method" do
+    @db.extend_datasets{exclude_having :released, :released}
+    ds.released.sql.must_equal 'SELECT * FROM items HAVING NOT released'
+    ds.where(:foo).released.sql.must_equal 'SELECT * FROM items WHERE foo HAVING NOT released'
+  end
+
+  it "should have dataset_module support a distinct method" do
+    @db.extend_datasets{def supports_distinct_on?; true end; distinct :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT DISTINCT ON (baz) * FROM items'
+    ds.where(:bar).foo.sql.must_equal 'SELECT DISTINCT ON (baz) * FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a grep method" do
+    @db.extend_datasets{grep :foo, :baz, 'quux%'}
+    ds.foo.sql.must_equal 'SELECT * FROM items WHERE ((baz LIKE \'quux%\' ESCAPE \'\\\'))'
+    ds.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE (bar AND ((baz LIKE \'quux%\' ESCAPE \'\\\')))'
+  end
+
+  it "should have dataset_module support a group method" do
+    @db.extend_datasets{group :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT * FROM items GROUP BY baz'
+    ds.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a group_and_count method" do
+    @db.extend_datasets{group_and_count :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT baz, count(*) AS count FROM items GROUP BY baz'
+    ds.where(:bar).foo.sql.must_equal 'SELECT baz, count(*) AS count FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a group_append method" do
+    @db.extend_datasets{group_append :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT * FROM items GROUP BY baz'
+    ds.group(:bar).foo.sql.must_equal 'SELECT * FROM items GROUP BY bar, baz'
+  end
+
+  it "should have dataset_module support a limit method" do
+    @db.extend_datasets{limit :foo, 1}
+    ds.foo.sql.must_equal 'SELECT * FROM items LIMIT 1'
+    ds.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar LIMIT 1'
+  end
+
+  it "should have dataset_module support a offset method" do
+    @db.extend_datasets{offset :foo, 1}
+    ds.foo.sql.must_equal 'SELECT * FROM items OFFSET 1'
+    ds.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar OFFSET 1'
+  end
+
+  it "should have dataset_module support a order method" do
+    @db.extend_datasets{order(:foo){:baz}}
+    ds.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    ds.where(:bar).foo.sql.must_equal 'SELECT * FROM items WHERE bar ORDER BY baz'
+  end
+
+  it "should have dataset_module support a order_append method" do
+    @db.extend_datasets{order_append :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    ds.order(:bar).foo.sql.must_equal 'SELECT * FROM items ORDER BY bar, baz'
+  end
+
+  it "should have dataset_module support a order_prepend method" do
+    @db.extend_datasets{order_prepend :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT * FROM items ORDER BY baz'
+    ds.order(:bar).foo.sql.must_equal 'SELECT * FROM items ORDER BY baz, bar'
+  end
+
+  it "should have dataset_module support a select method" do
+    @db.extend_datasets{select :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT baz FROM items'
+    ds.where(:bar).foo.sql.must_equal 'SELECT baz FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_all method" do
+    @db.extend_datasets{select_all :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT baz.* FROM items'
+    ds.where(:bar).foo.sql.must_equal 'SELECT baz.* FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_append method" do
+    @db.extend_datasets{select_append :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT *, baz FROM items'
+    ds.where(:bar).foo.sql.must_equal 'SELECT *, baz FROM items WHERE bar'
+  end
+
+  it "should have dataset_module support a select_group method" do
+    @db.extend_datasets{select_group :foo, :baz}
+    ds.foo.sql.must_equal 'SELECT baz FROM items GROUP BY baz'
+    ds.where(:bar).foo.sql.must_equal 'SELECT baz FROM items WHERE bar GROUP BY baz'
+  end
+
+  it "should have dataset_module support a server method" do
+    @db.extend_datasets{server :foo, :baz}
+    ds.foo.opts[:server].must_equal :baz
+    ds.where(:bar).foo.opts[:server].must_equal :baz
+  end
+end
+
 describe "Database#disconnect_connection" do
   it "should call close on the connection" do
     o = Object.new
@@ -502,7 +610,7 @@ end
 describe "Database#synchronize" do
   before do
     @db = Sequel::Database.new(:max_connections => 1)
-    meta_def(@db, :connect){|c| 12345}
+    @db.define_singleton_method(:connect){|c| 12345}
   end
   
   it "should wrap the supplied block in pool.hold" do
@@ -527,7 +635,7 @@ describe "Database#test_connection" do
   before do
     @db = Sequel::Database.new
     pr = proc{@test = rand(100)}
-    meta_def(@db, :connect){|c| pr.call}
+    @db.define_singleton_method(:connect){|c| pr.call}
   end
   
   it "should attempt to get a connection" do
@@ -573,7 +681,7 @@ DatabaseTransactionSpecs = shared_description do
   end
   
   it "should support transaction isolation levels" do
-    meta_def(@db, :supports_transaction_isolation_levels?){true}
+    @db.define_singleton_method(:supports_transaction_isolation_levels?){true}
     [:uncommitted, :committed, :repeatable, :serializable].each do |l|
       @db.transaction(:isolation=>l){@db.run "DROP TABLE #{l}"}
     end
@@ -584,7 +692,7 @@ DatabaseTransactionSpecs = shared_description do
   end
 
   it "should allow specifying a default transaction isolation level" do
-    meta_def(@db, :supports_transaction_isolation_levels?){true}
+    @db.define_singleton_method(:supports_transaction_isolation_levels?){true}
     [:uncommitted, :committed, :repeatable, :serializable].each do |l|
       @db.transaction_isolation_level = l
       @db.transaction{@db.run "DROP TABLE #{l}"}
@@ -675,8 +783,8 @@ DatabaseTransactionSpecs = shared_description do
   
   it "should handle errors when sending BEGIN" do
     ec = Class.new(StandardError)
-    meta_def(@db, :database_error_classes){[ec]}
-    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /BEGIN/ ? raise(ec, 'bad') : super(c, sql)}
+    @db.define_singleton_method(:database_error_classes){[ec]}
+    @db.define_singleton_method(:log_connection_execute){|c, sql| sql =~ /BEGIN/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{@db.execute 'DROP TABLE test;'}
     rescue Sequel::DatabaseError => e
@@ -688,8 +796,8 @@ DatabaseTransactionSpecs = shared_description do
   
   it "should handle errors when sending COMMIT" do
     ec = Class.new(StandardError)
-    meta_def(@db, :database_error_classes){[ec]}
-    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /COMMIT/ ? raise(ec, 'bad') : super(c, sql)}
+    @db.define_singleton_method(:database_error_classes){[ec]}
+    @db.define_singleton_method(:log_connection_execute){|c, sql| sql =~ /COMMIT/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{@db.execute 'DROP TABLE test;'}
     rescue Sequel::DatabaseError => e
@@ -701,8 +809,8 @@ DatabaseTransactionSpecs = shared_description do
   
   it "should raise original exception if there is an exception raised when rolling back" do
     ec = Class.new(StandardError)
-    meta_def(@db, :database_error_classes){[ec]}
-    meta_def(@db, :log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
+    @db.define_singleton_method(:database_error_classes){[ec]}
+    @db.define_singleton_method(:log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
     begin
       @db.transaction{raise ArgumentError, 'asdf'}
     rescue => e
@@ -711,6 +819,18 @@ DatabaseTransactionSpecs = shared_description do
     @db.sqls.must_equal ['BEGIN']
   end
   
+  it "should raise original exception if there is an exception raised when rolling back when using :rollback=>:always" do
+    ec = Class.new(StandardError)
+    @db.define_singleton_method(:database_error_classes){[ec]}
+    @db.define_singleton_method(:log_connection_execute){|c, sql| sql =~ /ROLLBACK/ ? raise(ec, 'bad') : super(c, sql)}
+    begin
+      @db.transaction(:rollback=>:always){}
+    rescue => e
+    end
+    e.must_be_kind_of(ec)
+    @db.sqls.must_equal ['BEGIN']
+  end
+  
   it "should issue ROLLBACK if Sequel::Rollback is called in the transaction" do
     @db.transaction do
       @db.drop_table(:a)
@@ -794,10 +914,10 @@ DatabaseTransactionSpecs = shared_description do
   end
 
   it "should raise database errors when commiting a transaction as Sequel::DatabaseError" do
-    meta_def(@db, :commit_transaction){raise ArgumentError}
+    @db.define_singleton_method(:commit_transaction){raise ArgumentError}
     lambda{@db.transaction{}}.must_raise(ArgumentError)
 
-    meta_def(@db, :database_error_classes){[ArgumentError]}
+    @db.define_singleton_method(:database_error_classes){[ArgumentError]}
     lambda{@db.transaction{}}.must_raise(Sequel::DatabaseError)
   end
   
@@ -822,7 +942,7 @@ DatabaseTransactionSpecs = shared_description do
     tr.must_be :empty?
   end
 
-  it "should correctly handle nested transacation use with separate shards" do
+  it "should correctly handle nested transaction use with separate shards" do
     @db.transaction do |c1|
       @db.transaction(:server=>:test) do |c2|
         c1.wont_equal c2
@@ -924,13 +1044,13 @@ DatabaseTransactionSpecs = shared_description do
   end
 
   it "should raise an error if you attempt to use after_commit inside a prepared transaction" do
-    meta_def(@db, :supports_prepared_transactions?){true}
+    @db.define_singleton_method(:supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.after_commit{@db.execute('foo')}}}.must_raise(Sequel::Error)
     @db.sqls.must_equal ['BEGIN', 'ROLLBACK']
   end
 
   it "should raise an error if you attempt to use after_rollback inside a prepared transaction" do
-    meta_def(@db, :supports_prepared_transactions?){true}
+    @db.define_singleton_method(:supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.after_rollback{@db.execute('foo')}}}.must_raise(Sequel::Error)
     @db.sqls.must_equal ['BEGIN', 'ROLLBACK']
   end
@@ -996,13 +1116,13 @@ describe "Database#transaction with savepoint support" do
   end
 
   it "should raise an error if you attempt to use after_commit inside a savepoint in a prepared transaction" do
-    meta_def(@db, :supports_prepared_transactions?){true}
+    @db.define_singleton_method(:supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.transaction(:savepoint=>true){@db.after_commit{@db.execute('foo')}}}}.must_raise(Sequel::Error)
     @db.sqls.must_equal ['BEGIN', 'SAVEPOINT autopoint_1','ROLLBACK TO SAVEPOINT autopoint_1', 'ROLLBACK']
   end
 
   it "should raise an error if you attempt to use after_rollback inside a savepoint in a prepared transaction" do
-    meta_def(@db, :supports_prepared_transactions?){true}
+    @db.define_singleton_method(:supports_prepared_transactions?){true}
     proc{@db.transaction(:prepare=>'XYZ'){@db.transaction(:savepoint=>true){@db.after_rollback{@db.execute('foo')}}}}.must_raise(Sequel::Error)
     @db.sqls.must_equal ['BEGIN', 'SAVEPOINT autopoint_1','ROLLBACK TO SAVEPOINT autopoint_1', 'ROLLBACK']
   end
@@ -1021,7 +1141,7 @@ end
 describe "Database#transaction without savepoint support" do
   before do
     @db = Sequel.mock(:servers=>{:test=>{}})
-    meta_def(@db, :supports_savepoints?){false}
+    @db.define_singleton_method(:supports_savepoints?){false}
   end
 
   it "should not create savepoint if inside a transaction when :savepoint=>:only is used" do
@@ -1185,11 +1305,11 @@ describe "Database#transaction with savepoints" do
   end
   
   it "should raise database errors when commiting a transaction as Sequel::DatabaseError" do
-    meta_def(@db, :commit_transaction){raise ArgumentError}
+    @db.define_singleton_method(:commit_transaction){raise ArgumentError}
     lambda{@db.transaction{}}.must_raise(ArgumentError)
     lambda{@db.transaction{@db.transaction(:savepoint=>true){}}}.must_raise(ArgumentError)
 
-    meta_def(@db, :database_error_classes){[ArgumentError]}
+    @db.define_singleton_method(:database_error_classes){[ArgumentError]}
     lambda{@db.transaction{}}.must_raise(Sequel::DatabaseError)
     lambda{@db.transaction{@db.transaction(:savepoint=>true){}}}.must_raise(Sequel::DatabaseError)
   end
@@ -1197,7 +1317,7 @@ end
 
 describe "A Database adapter with a scheme" do
   before do
-    require 'sequel/adapters/mock'
+    require_relative '../../lib/sequel/adapters/mock'
     @ccc = Class.new(Sequel::Mock::Database)
     @ccc.send(:set_adapter_scheme, :ccc)
   end
@@ -1255,7 +1375,7 @@ describe "A Database adapter with a scheme" do
     Sequel.send(:def_adapter_method, :ccc)
     Sequel.ccc('db', :host=>'localhost', &p).must_equal returnValue
     @ccc::DISCONNECTS.must_equal [z, y, x]
-    class << Sequel; remove_method(:ccc) end
+    Sequel.singleton_class.send(:remove_method, :ccc)
   end
 
   it "should be accessible through Sequel.<adapter>" do
@@ -1280,7 +1400,7 @@ describe "A Database adapter with a scheme" do
     c = Sequel.ccc(:database => 'mydb', :host => 'localhost')
     c.must_be_kind_of(@ccc)
     c.opts.values_at(:adapter, :database, :host, :adapter_class).must_equal [:ccc, 'mydb', 'localhost', @ccc]
-    class << Sequel; remove_method(:ccc) end
+    Sequel.singleton_class.send(:remove_method, :ccc)
   end
   
   it "should be accessible through Sequel.connect with options" do
@@ -1375,7 +1495,7 @@ end
 
 describe "A single threaded database" do
   after do
-    Sequel::Database.single_threaded = false
+    Sequel.single_threaded = false
   end
   
   it "should use a SingleConnectionPool instead of a ConnectionPool" do
@@ -1388,14 +1508,9 @@ describe "A single threaded database" do
     db.pool.must_be_kind_of(Sequel::SingleConnectionPool)
   end
   
-  it "should be constructable using Database.single_threaded = true" do
-    Sequel::Database.single_threaded = true
-    db = Sequel::Database.new{123}
-    db.pool.must_be_kind_of(Sequel::SingleConnectionPool)
-  end
-
   it "should be constructable using Sequel.single_threaded = true" do
     Sequel.single_threaded = true
+    Sequel.single_threaded.must_equal true
     db = Sequel::Database.new{123}
     db.pool.must_be_kind_of(Sequel::SingleConnectionPool)
   end
@@ -1405,7 +1520,7 @@ describe "A single threaded database" do
   before do
     conn = 1234567
     @db = Sequel::Database.new(:single_threaded => true)
-    meta_def(@db, :connect) do |c|
+    @db.define_singleton_method(:connect) do |c|
       conn += 1
     end
   end
@@ -1426,21 +1541,19 @@ describe "A single threaded database" do
   end
   
   it "should convert an Exception on connection into a DatabaseConnectionError" do
-    db = Sequel::Database.new(:single_threaded => true, :servers=>{})
-    def db.connect(*) raise Exception end
+    db = Class.new(Sequel::Database){def connect(*) raise Exception end}.new(:single_threaded => true, :servers=>{}, :test=>false)
     proc {db.pool.hold {|c|}}.must_raise(Sequel::DatabaseConnectionError)
   end
   
   it "should raise a DatabaseConnectionError if the connection proc returns nil" do
-    db = Sequel.mock(:single_threaded => true, :servers=>{})
-    def db.connect(*) end
+    db = Class.new(Sequel::Database){def connect(*) end}.new(:single_threaded => true, :servers=>{}, :test=>false)
     proc {db.pool.hold {|c|}}.must_raise(Sequel::DatabaseConnectionError)
   end
 end
 
 describe "A database" do
   after do
-    Sequel::Database.single_threaded = false
+    Sequel.single_threaded = false
   end
   
   it "should have single_threaded? respond to true if in single threaded mode" do
@@ -1453,7 +1566,7 @@ describe "A database" do
     db = Sequel::Database.new
     db.wont_be :single_threaded?
     
-    Sequel::Database.single_threaded = true
+    Sequel.single_threaded = true
     
     db = Sequel::Database.new{123}
     db.must_be :single_threaded?
@@ -1553,11 +1666,11 @@ describe "Database#inspect" do
   end
 
   it "should include the class name and the connection options if an options hash was given" do
-    Sequel.connect(:adapter=>:mock).inspect.must_match(/#<Sequel::Mock::Database: \{:adapter=>:mock\}>/)
+    Sequel.connect(:adapter=>:mock).inspect.must_equal '#<Sequel::Mock::Database: {:adapter=>:mock}>'
   end
 
   it "should include the class name, uri, and connection options if uri and options hash was given" do
-    Sequel.connect('mock://foo', :database=>'bar').inspect.must_match(/#<Sequel::Mock::Database: "mock:\/\/foo" \{:database=>"bar"\}>/)
+    Sequel.connect('mock://foo', :database=>'bar').inspect.must_equal '#<Sequel::Mock::Database: "mock://foo" {:database=>"bar"}>'
   end
 end
 
@@ -1667,6 +1780,10 @@ describe "Database#add_servers" do
     @db.synchronize(:server1){|c| c.opts[:host].must_equal 8}
     @db.synchronize(:server2){|c| c.opts[:host].must_equal 4}
   end
+
+  it "should raise error for unsharded pool" do
+    proc{Sequel.mock.add_servers(:server1=>{})}.must_raise Sequel::Error
+  end
 end
 
 describe "Database#remove_servers" do
@@ -1717,66 +1834,12 @@ describe "Database#remove_servers" do
       c1.opts[:host].must_equal 1
     end
   end
-end
 
-describe "Database#each_server with do/jdbc adapter connection string without :adapter option" do
-  it "should yield a separate database object for each server" do
-    require 'sequel/adapters/mock'
-    klass = Class.new(Sequel::Database)
-    def klass.adapter_class(v)
-      raise unless v == :jdbc
-      Sequel::Mock::Database
-    end
-    @db = klass.connect('jdbc:blah:', :host=>1, :database=>2, :servers=>{:server1=>{:host=>3}})
-
-    hosts = []
-    @db.each_server do |db|
-      db.must_be_kind_of(Sequel::Database)
-      db.wont_equal @db
-      db.opts[:adapter_class].must_equal Sequel::Mock::Database
-      db.opts[:database].must_equal 2
-      hosts << db.opts[:host]
-    end
-    hosts.sort.must_equal [1, 3]
-  end
-
-  it "should raise if not given a block" do
-    proc{Sequel.mock.each_server}.must_raise(Sequel::Error)
+  it "should raise error for unsharded pool" do
+    proc{Sequel.mock.remove_servers(:server1)}.must_raise Sequel::Error
   end
 end
 
-describe "Database#each_server" do
-  before do
-    @db = Sequel.mock(:host=>1, :database=>2, :servers=>{:server1=>{:host=>3}, :server2=>{:host=>4}})
-  end
-
-  it "should yield a separate database object for each server" do
-    hosts = []
-    @db.each_server do |db|
-      db.must_be_kind_of(Sequel::Database)
-      db.wont_equal @db
-      db.opts[:adapter].must_equal :mock
-      db.opts[:database].must_equal 2
-      hosts << db.opts[:host]
-    end
-    hosts.sort.must_equal [1, 3, 4]
-  end
-
-  it "should disconnect and remove entry from Sequel::DATABASES after use" do
-    dbs = []
-    dcs = []
-    @db.each_server do |db|
-      dbs << db
-      Sequel::DATABASES.must_include(db)
-      meta_def(db, :disconnect){dcs << db}
-    end
-    dbs.each do |db|
-      Sequel::DATABASES.wont_include(db)
-    end
-    dbs.must_equal dcs
-  end
-end
-  
 describe "Database#raise_error" do
   before do
     @db = Sequel.mock
@@ -1913,9 +1976,9 @@ describe "Database#typecast_value" do
     t2 = Time.mktime(2011, 1, 2, 3, 4, 5, 500000) # Local Time
     t3 = Time.utc(2011, 1, 2, 3, 4, 5, 500000) - (t - t2) # Local Time in UTC Time
     t4 = Time.mktime(2011, 1, 2, 3, 4, 5, 500000) + (t - t2) # UTC Time in Local Time
-    secs = defined?(Rational) ? Rational(11, 2) : 5.5
-    r1 = defined?(Rational) ? Rational(t2.utc_offset, 86400) : t2.utc_offset/86400.0
-    r2 = defined?(Rational) ? Rational((t - t2).to_i, 86400) : (t - t2).to_i/86400.0
+    secs = Rational(11, 2)
+    r1 = Rational(t2.utc_offset, 86400)
+    r2 = Rational((t - t2).to_i, 86400)
     dt = DateTime.civil(2011, 1, 2, 3, 4, secs)
     dt2 = DateTime.civil(2011, 1, 2, 3, 4, secs, r1)
     dt3 = DateTime.civil(2011, 1, 2, 3, 4, secs) - r2
@@ -2054,8 +2117,8 @@ describe "Database#typecast_value" do
 
       Sequel.datetime_class = DateTime
       @db.typecast_value(:datetime, [2011, 10, 11, 12, 13, 14]).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14)
-      @db.typecast_value(:datetime, [2011, 10, 11, 12, 13, 14, 500000000]).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5))
-      @db.typecast_value(:datetime, [2011, 10, 11, 12, 13, 14, 500000000, (defined?(Rational) ? Rational(1, 2) : 0.5)]).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5), (defined?(Rational) ? Rational(1, 2) : 0.5))
+      @db.typecast_value(:datetime, [2011, 10, 11, 12, 13, 14, 500000000]).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2))
+      @db.typecast_value(:datetime, [2011, 10, 11, 12, 13, 14, 500000000, Rational(1, 2)]).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2), Rational(1, 2))
     ensure
       Sequel.datetime_class = Time
     end
@@ -2070,13 +2133,13 @@ describe "Database#typecast_value" do
 
       Sequel.datetime_class = DateTime
       @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14)
-      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :nanos=>500000000).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5))
+      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :nanos=>500000000).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2))
       @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14)
-      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'nanos'=>500000000).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5))
-      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :offset=>(defined?(Rational) ? Rational(1, 2) : 0.5)).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14, (defined?(Rational) ? Rational(1, 2) : 0.5))
-      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :nanos=>500000000, :offset=>(defined?(Rational) ? Rational(1, 2) : 0.5)).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5), (defined?(Rational) ? Rational(1, 2) : 0.5))
-      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'offset'=>(defined?(Rational) ? Rational(1, 2) : 0.5)).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14, (defined?(Rational) ? Rational(1, 2) : 0.5))
-      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'nanos'=>500000000, 'offset'=>(defined?(Rational) ? Rational(1, 2) : 0.5)).must_equal DateTime.civil(2011, 10, 11, 12, 13, (defined?(Rational) ? Rational(29, 2) : 14.5), (defined?(Rational) ? Rational(1, 2) : 0.5))
+      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'nanos'=>500000000).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2))
+      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :offset=>Rational(1, 2)).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14, Rational(1, 2))
+      @db.typecast_value(:datetime, :year=>2011, :month=>10, :day=>11, :hour=>12, :minute=>13, :second=>14, :nanos=>500000000, :offset=>Rational(1, 2)).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2), Rational(1, 2))
+      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'offset'=>Rational(1, 2)).must_equal DateTime.civil(2011, 10, 11, 12, 13, 14, Rational(1, 2))
+      @db.typecast_value(:datetime, 'year'=>2011, 'month'=>10, 'day'=>11, 'hour'=>12, 'minute'=>13, 'second'=>14, 'nanos'=>500000000, 'offset'=>Rational(1, 2)).must_equal DateTime.civil(2011, 10, 11, 12, 13, Rational(29, 2), Rational(1, 2))
     ensure
       Sequel.datetime_class = Time
     end
@@ -2138,12 +2201,37 @@ describe "Database#typecast_value" do
     end
   end
 
+  it "should have an underlying exception class available at cause" do
+    begin
+      @db.typecast_value(:date, 'a')
+      true.must_equal false
+    rescue Sequel::InvalidValue => e
+      e.cause.must_be_kind_of(ArgumentError)
+    end
+  end if RUBY_VERSION >= '2.1'
+
+  it "should have an underlying exception class available at cause when using nested exceptions" do
+    begin
+      begin
+        raise ArgumentError
+      rescue => e1
+        begin
+          raise RuntimeError
+        rescue
+          @db.send(:raise_error, e1)
+        end
+      end
+    rescue Sequel::DatabaseError => e
+      e.cause.must_be_kind_of(ArgumentError)
+    end
+  end if RUBY_VERSION >= '2.1'
+
   it "should include underlying exception class in #inspect" do
     begin
       @db.typecast_value(:date, 'a')
       true.must_equal false
     rescue Sequel::InvalidValue => e
-      e.inspect.must_match(/\A#<Sequel::InvalidValue: ArgumentError: .*>\z/)
+      e.inspect.must_equal '#<Sequel::InvalidValue: ArgumentError: invalid date>'
     end
   end
 end
@@ -2291,8 +2379,8 @@ describe "Database#supports_savepoints_in_prepared_transactions?" do
 
   it "should be true if both savepoints and prepared transactions are supported" do
     db = Sequel::Database.new
-    meta_def(db, :supports_savepoints?){true}
-    meta_def(db, :supports_prepared_transactions?){true}
+    db.define_singleton_method(:supports_savepoints?){true}
+    db.define_singleton_method(:supports_prepared_transactions?){true}
     db.supports_savepoints_in_prepared_transactions?.must_equal true
   end
 end
@@ -2401,7 +2489,7 @@ describe "Database extensions" do
     end
   end
   before do
-    @db = Sequel.mock(:identifier_mangling=>false)
+    @db = Sequel.mock
   end
   after do
     Sequel::Database.instance_variable_set(:@initialize_hook, Proc.new {|db| })
@@ -2456,8 +2544,8 @@ describe "Database extensions" do
     Sequel::Database.extension(:foo, :bar)
     @db.wont_respond_to(:a)
     @db.wont_respond_to(:b)
-    Sequel.mock(:identifier_mangling=>false).a.must_equal 1
-    Sequel.mock(:identifier_mangling=>false).b.must_equal 2
+    Sequel.mock.a.must_equal 1
+    Sequel.mock.b.must_equal 2
   end
 end
 
@@ -2548,6 +2636,6 @@ end
 
 describe "Dataset identifier folding" do
   it "should fold to uppercase by default, as per SQL" do
-    Sequel::Database.new(:identifier_mangling=>false).send(:folds_unquoted_identifiers_to_uppercase?).must_equal true
+    Sequel::Database.new.send(:folds_unquoted_identifiers_to_uppercase?).must_equal true
   end
 end
