@@ -1,9 +1,9 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper')
+require_relative "spec_helper"
 
-describe "Sequel::Schema::Generator dump methods" do
+describe "Sequel::Schema::CreateTableGenerator dump methods" do
   before do
     @d = Sequel::Database.new.extension(:schema_dumper)
-    @g = Sequel::Schema::Generator
+    @g = Sequel::Schema::CreateTableGenerator
   end
 
   it "should allow the same table information to be converted to a string for evaling inside of another instance with the same result" do
@@ -75,9 +75,9 @@ end
 describe "Sequel::Database dump methods" do
   before do
     @d = Sequel::Database.new.extension(:schema_dumper)
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:schema) do |t, *o|
-      case t
+    def @d.tables(o) o[:schema] ? [o[:schema]] : [:t1, :t2] end
+    def @d.schema(t, *o)
+      v = case t
       when :t1, 't__t1', Sequel.identifier(:t__t1)
         [[:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true, :allow_null=>false}],
          [:c2, {:db_type=>'varchar(20)', :allow_null=>true}]]
@@ -90,9 +90,19 @@ describe "Sequel::Database dump methods" do
       when :t5
         [[:c1, {:db_type=>'blahblah', :allow_null=>true}]]
       end
+
+      if o.first.is_a?(Hash) && o.first[:schema]
+        v.last.last[:db_type] = o.first[:schema]
+      end
+
+      v
     end
   end
 
+  it "should support dumping table with :schema option" do
+    @d.dump_table_schema(:t1, :schema=>'varchar(15)').must_equal "create_table(:t1) do\n  primary_key :c1\n  String :c2, :size=>15\nend"
+  end
+
   it "should support dumping table schemas as create_table method calls" do
     @d.dump_table_schema(:t1).must_equal "create_table(:t1) do\n  primary_key :c1\n  String :c2, :size=>20\nend"
   end
@@ -106,25 +116,30 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should dump non-Integer primary key columns with explicit :type" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'bigint', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'bigint', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]] end
     @d.dump_table_schema(:t6).must_equal "create_table(:t6) do\n  primary_key :c1, :type=>:Bignum\nend"
   end
 
+  it "should dump non-Integer primary key columns with explicit :type when using :same_db=>true" do
+    def @d.schema(*s) [[:c1, {:db_type=>'bigint', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]] end
+    @d.dump_table_schema(:t6, :same_db=>true).must_equal "create_table(:t6) do\n  primary_key :c1, :type=>:Bignum\nend"
+  end
+
   it "should dump auto incrementing primary keys with :keep_order option if they are not first" do
     @d.dump_table_schema(:t3).must_equal "create_table(:t3) do\n  String :c2, :size=>20\n  primary_key :c1, :keep_order=>true\nend"
   end
 
   it "should handle foreign keys" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :allow_null=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}] end
     @d.dump_table_schema(:t6).must_equal "create_table(:t6) do\n  foreign_key :c1, :t2, :key=>[:c2]\nend"
   end
 
   it "should handle primary keys that are also foreign keys" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2]}] end
     s = @d.dump_table_schema(:t6)
     s.must_match(/create_table\(:t6\) do\n  primary_key :c1, /)
     s.must_match(/:table=>:t2/)
@@ -132,9 +147,9 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should handle foreign key options" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :allow_null=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}] end
     s = @d.dump_table_schema(:t6)
     s.must_match(/create_table\(:t6\) do\n  foreign_key :c1, :t2, /)
     s.must_match(/:key=>\[:c2\]/)
@@ -144,9 +159,9 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should handle foreign key options in the primary key" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:restrict, :on_update=>:set_null, :deferrable=>true}] end
     s = @d.dump_table_schema(:t6)
     s.must_match(/create_table\(:t6\) do\n  primary_key :c1, /)
     s.must_match(/:table=>:t2/)
@@ -157,9 +172,9 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should omit foreign key options that are the same as defaults" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :allow_null=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :allow_null=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}] end
     s = @d.dump_table_schema(:t6)
     s.must_match(/create_table\(:t6\) do\n  foreign_key :c1, :t2, /)
     s.must_match(/:key=>\[:c2\]/)
@@ -169,9 +184,9 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should omit foreign key options that are the same as defaults in the primary key" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :primary_key=>true, :allow_null=>true, :auto_increment=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1], :table=>:t2, :key=>[:c2], :on_delete=>:no_action, :on_update=>:no_action, :deferrable=>false}] end
     s = @d.dump_table_schema(:t6)
     s.must_match(/create_table\(:t6\) do\n  primary_key :c1, /)
     s.must_match(/:table=>:t2/)
@@ -182,7 +197,7 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should dump primary key columns with explicit type equal to the database type when :same_db option is passed" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'somedbspecifictype', :primary_key=>true, :allow_null=>false}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'somedbspecifictype', :primary_key=>true, :allow_null=>false}]] end
     @d.dump_table_schema(:t7, :same_db => true).must_equal "create_table(:t7) do\n  column :c1, \"somedbspecifictype\", :null=>false\n  \n  primary_key [:c1]\nend"
   end
 
@@ -191,21 +206,34 @@ describe "Sequel::Database dump methods" do
   end
 
   it "should use a composite foreign_key calls if there is a composite foreign key" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer'}], [:c2, {:db_type=>'integer'}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|*s| [{:columns=>[:c1, :c2], :table=>:t2, :key=>[:c3, :c4]}]}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer'}], [:c2, {:db_type=>'integer'}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(*s) [{:columns=>[:c1, :c2], :table=>:t2, :key=>[:c3, :c4]}] end
     @d.dump_table_schema(:t1).must_equal "create_table(:t1) do\n  Integer :c1\n  Integer :c2\n  \n  foreign_key [:c1, :c2], :t2, :key=>[:c3, :c4]\nend"
   end
 
   it "should include index information if available" do
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
     @d.dump_table_schema(:t1).must_equal "create_table(:t1, :ignore_index_errors=>true) do\n  primary_key :c1\n  String :c2, :size=>20\n  \n  index [:c1], :name=>:i1\n  index [:c2, :c1], :unique=>true\nend"
   end
 
+  it "should support dumping the whole database as a migration with a :schema option" do
+    @d.dump_schema_migration(:schema=>'t__t1').must_equal <<-END_MIG
+Sequel.migration do
+  change do
+    create_table("t__t1") do
+      primary_key :c1
+      String :c2
+    end
+  end
+end
+END_MIG
+  end
+
   it "should support dumping the whole database as a migration" do
     @d.dump_schema_migration.must_equal <<-END_MIG
 Sequel.migration do
@@ -227,7 +255,7 @@ END_MIG
   end
 
   it "should sort table names when dumping a migration" do
-    @d.meta_def(:tables){|o| [:t2, :t1]}
+    def @d.tables(o) [:t2, :t1] end
     @d.dump_schema_migration.must_equal <<-END_MIG
 Sequel.migration do
   change do
@@ -248,12 +276,12 @@ END_MIG
   end
 
   it "should sort table names topologically when dumping a migration with foreign keys" do
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:schema) do |t|
+    def @d.tables(o) [:t1, :t2] end
+    def @d.schema(t, *o)
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true}]]
     end
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list) do |t|
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
     @d.dump_schema_migration.must_equal <<-END_MIG
@@ -272,12 +300,12 @@ END_MIG
   end
 
   it "should handle circular dependencies when dumping a migration with foreign keys" do
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:schema) do |t|
+    def @d.tables(o) [:t1, :t2] end
+    def @d.schema(t, *o)
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer'}]]
     end
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list) do |t|
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : [{:columns=>[:c1], :table=>:t1, :key=>[:c2]}]
     end
     @d.dump_schema_migration.must_equal <<-END_MIG
@@ -300,12 +328,12 @@ END_MIG
   end
 
   it "should sort topologically even if the database raises an error when trying to parse foreign keys for a non-existent table" do
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:schema) do |t|
+    def @d.tables(o) [:t1, :t2] end
+    def @d.schema(t, *o)
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true}]]
     end
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list) do |t|
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t)
       raise Sequel::DatabaseError unless [:t1, :t2].include?(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
@@ -346,8 +374,8 @@ END_MIG
   end
 
   it "should honor the :index_names => false option to not include names of indexes" do
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
@@ -378,8 +406,8 @@ END_MIG
   end
   
   it "should make :index_names => :namespace option a noop if there is a  global index namespace" do
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
@@ -410,9 +438,9 @@ END_MIG
   end
 
   it "should honor the :index_names => :namespace option to include names of indexes with prepended table name if there is no global index namespace" do
-    @d.meta_def(:global_index_namespace?){false}
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.global_index_namespace?; false end
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
@@ -443,8 +471,8 @@ END_MIG
   end
 
   it "should honor the :indexes => false option to not include indexes" do
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
@@ -469,24 +497,24 @@ END_MIG
   end
 
   it "should have :indexes => false option disable foreign keys as well when dumping a whole migration" do
-    @d.meta_def(:foreign_key_list) do |t|
+    def @d.foreign_key_list(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
     @d.dump_schema_migration(:indexes=>false).wont_match(/foreign_key/)
   end
 
   it "should have :foreign_keys option override :indexes => false disabling of foreign keys" do
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list) do |t|
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t)
       t == :t1 ? [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}] : []
     end
     @d.dump_schema_migration(:indexes=>false, :foreign_keys=>true).must_match(/foreign_key/)
   end
 
   it "should support dumping just indexes as a migration" do
-    @d.meta_def(:tables){|o| [:t1]}
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.tables(o) [:t1] end
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
@@ -501,9 +529,9 @@ END_MIG
   end
 
   it "should honor the :index_names => false option to not include names of indexes when dumping just indexes as a migration" do
-    @d.meta_def(:tables){|o| [:t1]}
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.tables(o) [:t1] end
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>true}}
     end
@@ -518,9 +546,9 @@ END_MIG
   end
 
   it "should honor the :index_names => :namespace option be a noop if there is a global index namespace" do
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.tables(o) [:t1, :t2] end
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
@@ -538,10 +566,10 @@ END_MIG
   end
 
   it "should honor the :index_names => :namespace option to include names of indexes with prepended table name when dumping just indexes as a migration if there is no global index namespace" do
-    @d.meta_def(:global_index_namespace?){false}
-    @d.meta_def(:tables){|o| [:t1, :t2]}
-    @d.meta_def(:supports_index_parsing?){true}
-    @d.meta_def(:indexes) do |t|
+    def @d.global_index_namespace?; false end
+    def @d.tables(o) [:t1, :t2] end
+    def @d.supports_index_parsing?; true end
+    def @d.indexes(t)
       {:i1=>{:columns=>[:c1], :unique=>false},
        :t1_c2_c1_index=>{:columns=>[:c2, :c1], :unique=>false}}
     end
@@ -559,7 +587,7 @@ END_MIG
   end
 
   it "should handle missing index parsing support when dumping index migration" do
-    @d.meta_def(:tables){|o| [:t1]}
+    def @d.tables(o) [:t1] end
     @d.dump_indexes_migration.must_equal <<-END_MIG
 Sequel.migration do
   change do
@@ -570,7 +598,7 @@ END_MIG
   end
 
   it "should handle missing foreign key parsing support when dumping foreign key migration" do
-    @d.meta_def(:tables){|o| [:t1]}
+    def @d.tables(o) [:t1] end
     @d.dump_foreign_key_migration.must_equal <<-END_MIG
 Sequel.migration do
   change do
@@ -581,12 +609,12 @@ END_MIG
   end
 
   it "should support dumping just foreign_keys as a migration" do
-    @d.meta_def(:tables){|o| [:t1, :t2, :t3]}
-    @d.meta_def(:schema) do |t|
+    def @d.tables(o) [:t1, :t2, :t3] end
+    def @d.schema(t, *o)
       t == :t1 ? [[:c2, {:db_type=>'integer'}]] : [[:c1, {:db_type=>'integer'}]]
     end
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list) do |t, *a|
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t, *a)
       case t
       when :t1
         [{:columns=>[:c2], :table=>:t2, :key=>[:c1]}]
@@ -612,12 +640,12 @@ END_MIG
   end
 
   it "should handle not null values and defaults" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'date', :default=>"'now()'", :allow_null=>true}], [:c2, {:db_type=>'datetime', :allow_null=>false}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'date', :default=>"'now()'", :allow_null=>true}], [:c2, {:db_type=>'datetime', :allow_null=>false}]] end
     @d.dump_table_schema(:t3).must_equal "create_table(:t3) do\n  Date :c1\n  DateTime :c2, :null=>false\nend"
   end
   
   it "should handle converting common defaults" do
-    @d.meta_def(:schema) do |t, *os|
+    def @d.schema(t, *os)
       s = [[:c1, {:db_type=>'boolean', :default=>"false", :type=>:boolean, :allow_null=>true}],
        [:c2, {:db_type=>'varchar', :default=>"'blah'", :type=>:string, :allow_null=>true}],
        [:c3, {:db_type=>'integer', :default=>"-1", :type=>:integer, :allow_null=>true}],
@@ -639,10 +667,10 @@ END_MIG
   end
   
   it "should not use a literal string as a fallback if using MySQL with the :same_db option" do
-    @d.meta_def(:database_type){:mysql}
-    @d.meta_def(:supports_index_parsing?){false}
-    @d.meta_def(:supports_foreign_key_parsing?){false}
-    @d.meta_def(:schema) do |t, *os|
+    def @d.database_type; :mysql end
+    def @d.supports_index_parsing?; false end
+    def @d.supports_foreign_key_parsing?; false end
+    def @d.schema(t, *os)
       s = [[:c10, {:db_type=>'foo', :default=>"'6 weeks'", :type=>nil, :allow_null=>true}]]
       s.each{|_, c| c[:ruby_default] = column_schema_to_ruby_default(c[:default], c[:type])}
       s
@@ -655,7 +683,8 @@ END_MIG
   end
 
   it "should convert many database types to ruby types" do
-    types = %w"mediumint smallint int integer mediumint(6) smallint(7) int(8) integer(9)
+    def @d.schema(t, *o)
+      types = %w"mediumint smallint int integer mediumint(6) smallint(7) int(8) integer(9)
       tinyint tinyint(2) bigint bigint(20) real float double boolean tinytext mediumtext
       longtext text clob date datetime timestamp time char character
       varchar varchar(255) varchar(30) bpchar string money
@@ -667,7 +696,6 @@ END_MIG
       ["timestamp(6) without time zone", "timestamp(6) with time zone", 'mediumint(10) unsigned', 'int(9) unsigned',
        'int(10) unsigned', "int(12) unsigned", 'bigint unsigned', 'tinyint(3) unsigned', 'identity', 'int identity'] +
       %w"integer(10) bit bool"
-    @d.meta_def(:schema) do |t, *o|
       i = 0
       types.map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
     end
@@ -759,10 +787,9 @@ END_MIG
   end
 
   it "should convert mysql types to ruby types" do
-    types = ['double(15,2)', 'double(7,1) unsigned']
-    @d.meta_def(:schema) do |t, *o|
+    def @d.schema(t, *o)
       i = 0
-      types.map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
+      ['double(15,2)', 'double(7,1) unsigned'].map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>true}]}
     end
     @d.dump_table_schema(:x).must_equal((<<END_MIG).chomp)
 create_table(:x) do
@@ -774,25 +801,40 @@ end
 END_MIG
   end
 
+  it "should convert oracle special types to ruby types" do
+    def @d.database_type; :oracle end
+    def @d.schema(t, *o)
+      i = 0
+      ['number not null', 'date not null', 'varchar2(4 byte) not null'].map{|x| [:"c#{i+=1}", {:db_type=>x, :allow_null=>false}]}
+    end
+    @d.dump_table_schema(:x).must_equal((<<END_MIG).chomp)
+create_table(:x) do
+  BigDecimal :c1, :null=>false
+  Date :c2, :null=>false
+  String :c3, :null=>false
+end
+END_MIG
+  end
+
   it "should force specify :null option for MySQL timestamp columns when using :same_db" do
-    @d.meta_def(:database_type){:mysql}
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]]}
+    def @d.database_type; :mysql end
+    def @d.schema(*s) [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]] end
     @d.dump_table_schema(:t3, :same_db=>true).must_equal "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>true\n  \n  primary_key [:c1]\nend"
 
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>false}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>false}]] end
     @d.dump_table_schema(:t3, :same_db=>true).must_equal "create_table(:t3) do\n  column :c1, \"timestamp\", :null=>false\n  \n  primary_key [:c1]\nend"
   end
 
   it "should use separate primary_key call with non autoincrementable types" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'varchar(8)', :primary_key=>true, :auto_increment=>false}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'varchar(8)', :primary_key=>true, :auto_increment=>false}]] end
     @d.dump_table_schema(:t3).must_equal "create_table(:t3) do\n  String :c1, :size=>8\n  \n  primary_key [:c1]\nend"
     @d.dump_table_schema(:t3, :same_db=>true).must_equal "create_table(:t3) do\n  column :c1, \"varchar(8)\"\n  \n  primary_key [:c1]\nend"
   end
 
   it "should use explicit type for non integer foreign_key types" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'date', :primary_key=>true, :auto_increment=>false}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|t, *a| [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4}
+    def @d.schema(*s) [[:c1, {:db_type=>'date', :primary_key=>true, :auto_increment=>false}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t, *a) [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4 end
     ["create_table(:t4) do\n  foreign_key :c1, :t3, :type=>Date, :key=>[:c1]\n  \n  primary_key [:c1]\nend",
      "create_table(:t4) do\n  foreign_key :c1, :t3, :key=>[:c1], :type=>Date\n  \n  primary_key [:c1]\nend"].must_include(@d.dump_table_schema(:t4))
     ["create_table(:t4) do\n  foreign_key :c1, :t3, :type=>\"date\", :key=>[:c1]\n  \n  primary_key [:c1]\nend",
@@ -800,16 +842,16 @@ END_MIG
   end
 
   it "should correctly handing autoincrementing primary keys that are also foreign keys" do
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true}]]}
-    @d.meta_def(:supports_foreign_key_parsing?){true}
-    @d.meta_def(:foreign_key_list){|t, *a| [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4}
+    def @d.schema(*s) [[:c1, {:db_type=>'integer', :primary_key=>true, :auto_increment=>true}]] end
+    def @d.supports_foreign_key_parsing?; true end
+    def @d.foreign_key_list(t, *a) [{:columns=>[:c1], :table=>:t3, :key=>[:c1]}] if t == :t4 end
     ["create_table(:t4) do\n  primary_key :c1, :table=>:t3, :key=>[:c1]\nend",
      "create_table(:t4) do\n  primary_key :c1, :key=>[:c1], :table=>:t3\nend"].must_include(@d.dump_table_schema(:t4))
   end
 
   it "should handle dumping on PostgreSQL using qualified tables" do
     @d = Sequel.connect('mock://postgres').extension(:schema_dumper)
-    @d.meta_def(:schema){|*s| [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]]}
+    def @d.schema(*s) [[:c1, {:db_type=>'timestamp', :primary_key=>true, :allow_null=>true}]] end
     @d.dump_table_schema(Sequel.qualify(:foo, :bar), :same_db=>true).must_equal "create_table(Sequel::SQL::QualifiedIdentifier.new(:foo, :bar)) do\n  column :c1, \"timestamp\"\n  \n  primary_key [:c1]\nend"
   end
 end
