@@ -1,4 +1,4 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
+require_relative "spec_helper"
 
 describe "Class Table Inheritance Plugin" do
   before(:all) do
@@ -38,7 +38,8 @@ describe "Class Table Inheritance Plugin" do
     class ::Staff < Employee
       many_to_one :manager
     end 
-    
+    class ::Intern < Employee
+    end 
     
     @i1 = @db[:employees].insert(:name=>'E', :kind=>'Employee')
     @i2 = @db[:employees].insert(:name=>'S', :kind=>'Staff')
@@ -51,9 +52,10 @@ describe "Class Table Inheritance Plugin" do
     @db[:managers].insert(:id=>@i5, :num_staff=>2)
     @db[:executives].insert(:id=>@i5, :num_managers=>1)
     @db[:staff].insert(:id=>@i2, :manager_id=>@i4)
+    @i6 = @db[:employees].insert(:name=>'I', :kind=>'Intern')
   end
   after do
-    [:Ceo, :Executive, :Manager, :Staff, :Employee].each{|s| Object.send(:remove_const, s)}
+    [:Intern, :Ceo, :Executive, :Manager, :Staff, :Employee].each{|s| Object.send(:remove_const, s)}
   end
   after(:all) do
     @db.drop_table? :staff, :executives, :managers, :employees
@@ -65,7 +67,8 @@ describe "Class Table Inheritance Plugin" do
       Staff.load(:id=>@i2, :name=>'S', :kind=>'Staff'),
       Manager.load(:id=>@i3, :name=>'M', :kind=>'Manager'),
       Executive.load(:id=>@i4, :name=>'Ex', :kind=>'Executive'),
-      Ceo.load(:id=>@i5, :name=>'C', :kind=>'Ceo')
+      Ceo.load(:id=>@i5, :name=>'C', :kind=>'Ceo'),
+      Intern.load(:id=>@i6, :name=>'I', :kind=>'Intern'),
     ]
   end
   
@@ -102,6 +105,7 @@ describe "Class Table Inheritance Plugin" do
     Manager.db_schema.keys.sort_by{|x| x.to_s}.must_equal [:id, :kind, :name, :num_staff]
     Executive.db_schema.keys.sort_by{|x| x.to_s}.must_equal [:id, :kind, :name, :num_managers, :num_staff]
     Ceo.db_schema.keys.sort_by{|x| x.to_s}.must_equal [:id, :kind, :name, :num_managers, :num_staff]
+    Intern.db_schema.keys.sort_by{|x| x.to_s}.must_equal [:id, :kind, :name]
   end
   
   it "should include columns for tables for ancestor classes" do
@@ -110,6 +114,7 @@ describe "Class Table Inheritance Plugin" do
     Manager.columns.must_equal [:id, :name, :kind, :num_staff]
     Executive.columns.must_equal [:id, :name, :kind, :num_staff, :num_managers]
     Ceo.columns.must_equal [:id, :name, :kind, :num_staff, :num_managers]
+    Intern.columns.must_equal [:id, :name, :kind]
   end
   
   it "should delete rows from all tables" do
@@ -166,7 +171,7 @@ describe "Class Table Inheritance Plugin" do
   end
   
   it "should handle eagerly graphing one_to_many relationships" do
-    es = Executive.where(:name=>'Ex').eager_graph(:staff_members).all
+    es = Executive.where(Sequel[:employees][:name]=>'Ex').eager_graph(:staff_members).all
     es.must_equal [Executive[@i4]]
     es.map{|x| x.staff_members}.must_equal [[Staff[@i2]]]
   end
@@ -237,7 +242,6 @@ describe "Many Through Many Plugin" do
     Artist[@artist3.id].albums.map{|x| x.name}.sort.must_equal %w'B C'
     Artist[@artist4.id].albums.map{|x| x.name}.sort.must_equal %w'B D'
     
-    Artist.plugin :prepared_statements_associations
     Artist[@artist1.id].albums.map{|x| x.name}.sort.must_equal %w'A D'
     Artist[@artist2.id].albums.map{|x| x.name}.sort.must_equal %w'A C'
     Artist[@artist3.id].albums.map{|x| x.name}.sort.must_equal %w'B C'
@@ -294,13 +298,12 @@ describe "Many Through Many Plugin" do
   end
 
   it "should handle typical case with 3 join tables" do
-    Artist.many_through_many :related_artists, [[:albums_artists, :artist_id, :album_id], [:albums, :id, :id], [:albums_artists, :album_id, :artist_id]], :class=>Artist, :distinct=>true
+    Artist.many_through_many :related_artists, [[:albums_artists, :artist_id, :album_id], [:albums, :id, :id], [:albums_artists, :album_id, :artist_id]], :class=>Artist, :distinct=>true, :delay_pks=>false
     Artist[@artist1.id].related_artists.map{|x| x.name}.sort.must_equal %w'1 2 4'
     Artist[@artist2.id].related_artists.map{|x| x.name}.sort.must_equal %w'1 2 3'
     Artist[@artist3.id].related_artists.map{|x| x.name}.sort.must_equal %w'2 3 4'
     Artist[@artist4.id].related_artists.map{|x| x.name}.sort.must_equal %w'1 3 4'
     
-    Artist.plugin :prepared_statements_associations
     Artist[@artist1.id].related_artists.map{|x| x.name}.sort.must_equal %w'1 2 4'
     Artist[@artist2.id].related_artists.map{|x| x.name}.sort.must_equal %w'1 2 3'
     Artist[@artist3.id].related_artists.map{|x| x.name}.sort.must_equal %w'2 3 4'
@@ -366,7 +369,6 @@ describe "Many Through Many Plugin" do
     Artist[@artist3.id].related_albums.map{|x| x.name}.sort.must_equal %w'A B D'
     Artist[@artist4.id].related_albums.map{|x| x.name}.sort.must_equal %w'B D'
     
-    Artist.plugin :prepared_statements_associations
     Artist[@artist1.id].related_albums.map{|x| x.name}.sort.must_equal %w'A B C'
     Artist[@artist2.id].related_albums.map{|x| x.name}.sort.must_equal %w'A B C D'
     Artist[@artist3.id].related_albums.map{|x| x.name}.sort.must_equal %w'A B D'
@@ -568,7 +570,7 @@ describe "Touch plugin" do
     @album.updated_at.to_i.must_be_close_to Time.now.to_i, 2
   end
   
-  cspecify "should update the timestamp column for many_to_one associated records when the record is updated or destroyed", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+  cspecify "should update the timestamp column for many_to_one associated records when the record is updated or destroyed", [:jdbc, :sqlite] do
     Album.many_to_one :artist
     Album.plugin :touch, :associations=>:artist
     @artist.updated_at.must_be_nil
@@ -588,7 +590,7 @@ describe "Touch plugin" do
     end
   end
 
-  cspecify "should update the timestamp column for one_to_many associated records when the record is updated", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+  cspecify "should update the timestamp column for one_to_many associated records when the record is updated", [:jdbc, :sqlite] do
     Artist.one_to_many :albums
     Artist.plugin :touch, :associations=>:albums
     @album.updated_at.must_be_nil
@@ -601,7 +603,7 @@ describe "Touch plugin" do
     end
   end
 
-  cspecify "should update the timestamp column for many_to_many associated records when the record is updated", [:do, :sqlite], [:jdbc, :sqlite], [:swift] do
+  cspecify "should update the timestamp column for many_to_many associated records when the record is updated", [:jdbc, :sqlite] do
     Artist.many_to_many :albums
     Artist.plugin :touch, :associations=>:albums
     @artist.add_album(@album)
@@ -1193,11 +1195,12 @@ describe "AssociationPks plugin" do
     end
     class ::Artist < Sequel::Model
       plugin :association_pks
-      one_to_many :albums, :order=>:id
+      one_to_many :albums, :order=>:id, :delay_pks=>false
     end 
     class ::Album < Sequel::Model
       plugin :association_pks
-      many_to_many :tags, :order=>:id
+      many_to_many :tags, :order=>:id, :delay_pks=>false
+      many_to_many :uat_tags, :order=>Sequel[:tags][:id], :delay_pks=>false, :class=>:Tag, :right_key=>:tag_id, :association_pks_use_associated_table=>true
     end 
     class ::Tag < Sequel::Model
     end 
@@ -1263,6 +1266,10 @@ describe "AssociationPks plugin" do
     Album.order(:id).all.map{|a| a.tag_pks.sort}.must_equal [[@t1, @t2, @t3], [@t2], []]
   end
 
+  it "should return correct associated pks for many_to_many associations using :association_pks_use_associated_table" do
+    Album.order(:id).all.map{|a| a.uat_tag_pks.sort}.must_equal [[@t1, @t2, @t3], [@t2], []]
+  end
+
   it "should return correct associated right-side cpks for one_to_many associations" do
     Album.one_to_many :vocalists, :order=>:first
     Album.order(:id).all.map{|a| a.vocalist_pks.sort}.must_equal [[@v1, @v2, @v3], [], []]
@@ -1273,6 +1280,11 @@ describe "AssociationPks plugin" do
     Album.order(:id).all.map{|a| a.vocalist_pks.sort}.must_equal [[@v1, @v2, @v3], [@v2], []]
   end
 
+  it "should return correct associated right-side cpks for many_to_many associations when using :association_pks_use_associated_table" do
+    Album.many_to_many :vocalists, :join_table=>:albums_vocalists, :right_key=>[:first, :last], :order=>Sequel[:vocalists][:first], :association_pks_use_associated_table=>true
+    Album.order(:id).all.map{|a| a.vocalist_pks.sort}.must_equal [[@v1, @v2, @v3], [@v2], []]
+  end
+
   it "should return correct associated pks for left-side cpks for one_to_many associations" do
     Vocalist.one_to_many :instruments, :key=>[:first, :last], :order=>:id
     Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.must_equal [[@i1, @i2, @i3], [], []]
@@ -1283,6 +1295,11 @@ describe "AssociationPks plugin" do
     Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.must_equal [[@i1, @i2, @i3], [@i2], []]
   end
 
+  it "should return correct associated pks for left-side cpks for many_to_many associations when using :association_pks_use_associated_table" do
+    Vocalist.many_to_many :instruments, :join_table=>:vocalists_instruments, :left_key=>[:first, :last], :order=>:id, :association_pks_use_associated_table=>true
+    Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.must_equal [[@i1, @i2, @i3], [@i2], []]
+  end
+
   it "should return correct associated right-side cpks for left-side cpks for one_to_many associations" do
     Vocalist.one_to_many :hits, :key=>[:first, :last], :order=>:week
     Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.must_equal [[@h1, @h2, @h3], [], []]
@@ -1293,25 +1310,14 @@ describe "AssociationPks plugin" do
     Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.must_equal [[@h1, @h2, @h3], [@h2], []]
   end
 
-  it "should handle :delay association option for new instances" do
-    album_class = Class.new(Album)
-    album_class.many_to_many :tags, :clone=>:tags, :delay_pks=>true, :join_table=>:albums_tags, :left_key=>:album_id
-    album = album_class.new(:name=>'test album')
-    album.tag_pks.must_equal []
-    album.tag_pks = [@t1, @t2]
-    album.tag_pks.must_equal [@t1, @t2]
-    album.save
-    album_class.with_pk!(album.pk).tag_pks.sort.must_equal [@t1, @t2]
-
-    album.tag_pks = []
-    album.tag_pks.must_equal []
-    album.save
-    album_class.with_pk!(album.pk).tag_pks.sort.must_equal []
+  it "should return correct associated right-side cpks for left-side cpks for many_to_many associations when using :association_pks_use_associated_table" do
+    Vocalist.many_to_many :hits, :join_table=>:vocalists_hits, :left_key=>[:first, :last], :right_key=>[:year, :week], :order=>Sequel[:vocalists_hits][:week], :association_pks_use_associated_table=>true
+    Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.must_equal [[@h1, @h2, @h3], [@h2], []]
   end
 
-  it "should handle :delay=>:all association option for existing instances" do
+  it "should default to delaying association_pks setter method changes until saving" do
     album_class = Class.new(Album)
-    album_class.many_to_many :tags, :clone=>:tags, :delay_pks=>:always, :join_table=>:albums_tags, :left_key=>:album_id
+    album_class.many_to_many :tags, :clone=>:tags, :delay_pks=>true, :join_table=>:albums_tags, :left_key=>:album_id
     album = album_class.with_pk!(@al1)
     album.tag_pks.sort.must_equal [@t1, @t2, @t3]
     album.tag_pks = [@t1, @t2]
@@ -1368,7 +1374,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated right-side cpks correctly for a one_to_many association" do
     Album.use_transactions = true
-    Album.one_to_many :vocalists, :order=>:first
+    Album.one_to_many :vocalists, :order=>:first, :delay_pks=>false
     Album.order(:id).all.map{|a| a.vocalist_pks.sort}.must_equal [[@v1, @v2, @v3], [], []]
 
     Album[@al2].vocalist_pks = [@v1, @v3]
@@ -1389,7 +1395,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated right-side cpks correctly for a many_to_many association" do
     Album.use_transactions = true
-    Album.many_to_many :vocalists, :join_table=>:albums_vocalists, :right_key=>[:first, :last], :order=>:first
+    Album.many_to_many :vocalists, :join_table=>:albums_vocalists, :right_key=>[:first, :last], :order=>:first, :delay_pks=>false
 
     @db[:albums_vocalists].filter(:album_id=>@al1).select_order_map([:first, :last]).must_equal [@v1, @v2, @v3]
     Album[@al1].vocalist_pks = [@v1, @v3]
@@ -1412,7 +1418,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated pks correctly with left-side cpks for a one_to_many association" do
     Vocalist.use_transactions = true
-    Vocalist.one_to_many :instruments, :key=>[:first, :last], :order=>:id
+    Vocalist.one_to_many :instruments, :key=>[:first, :last], :order=>:id, :delay_pks=>false
     Vocalist.order(:first, :last).all.map{|a| a.instrument_pks.sort}.must_equal [[@i1, @i2, @i3], [], []]
 
     Vocalist[@v2].instrument_pks = [@i1, @i3]
@@ -1433,7 +1439,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated pks correctly with left-side cpks for a many_to_many association" do
     Vocalist.use_transactions = true
-    Vocalist.many_to_many :instruments, :join_table=>:vocalists_instruments, :left_key=>[:first, :last], :order=>:id
+    Vocalist.many_to_many :instruments, :join_table=>:vocalists_instruments, :left_key=>[:first, :last], :order=>:id, :delay_pks=>false
 
     @db[:vocalists_instruments].filter([:first, :last]=>[@v1]).select_order_map(:instrument_id).must_equal [@i1, @i2, @i3]
     Vocalist[@v1].instrument_pks = [@i1, @i3]
@@ -1456,7 +1462,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated right-side cpks correctly with left-side cpks for a one_to_many association" do
     Vocalist.use_transactions = true
-    Vocalist.one_to_many :hits, :key=>[:first, :last], :order=>:week
+    Vocalist.one_to_many :hits, :key=>[:first, :last], :order=>:week, :delay_pks=>false
     Vocalist.order(:first, :last).all.map{|a| a.hit_pks.sort}.must_equal [[@h1, @h2, @h3], [], []]
 
     Vocalist[@v2].hit_pks = [@h1, @h3]
@@ -1477,7 +1483,7 @@ describe "AssociationPks plugin" do
 
   it "should set associated right-side cpks correctly with left-side cpks for a many_to_many association" do
     Vocalist.use_transactions = true
-    Vocalist.many_to_many :hits, :join_table=>:vocalists_hits, :left_key=>[:first, :last], :right_key=>[:year, :week], :order=>:week
+    Vocalist.many_to_many :hits, :join_table=>:vocalists_hits, :left_key=>[:first, :last], :right_key=>[:year, :week], :order=>:week, :delay_pks=>false
 
     @db[:vocalists_hits].filter([:first, :last]=>[@v1]).select_order_map([:year, :week]).must_equal [@h1, @h2, @h3]
     Vocalist[@v1].hit_pks = [@h1, @h3]
@@ -1819,7 +1825,6 @@ describe "Sequel::Plugins::PreparedStatements" do
       Integer :i
     end
     @c = Class.new(Sequel::Model(@db[:ps_test]))
-    @c.plugin :prepared_statements_with_pk
   end
   before do
     @c.dataset.delete
@@ -1895,7 +1900,6 @@ describe "Sequel::Plugins::PreparedStatements with schema changes" do
     @c.one_to_many :ps_tests, :key=>:id, :class=>@c
     @c.many_to_many :mps_tests, :left_key=>:id, :right_key=>:id, :class=>@c, :join_table=>Sequel[:ps_test].as(:x)
     @c.plugin :prepared_statements
-    @c.plugin :prepared_statements_associations
   end
   after do
     @db.drop_table?(:ps_test)
@@ -1942,7 +1946,6 @@ describe "Caching plugins" do
   end
   before do
     @Album = Class.new(Sequel::Model(@db[:albums]))
-    @Album.plugin :many_to_one_pk_lookup
   end
   after(:all) do
     @db.drop_table?(:albums, :artists)
