@@ -1,50 +1,54 @@
-require File.join(File.dirname(File.expand_path(__FILE__)), 'spec_helper.rb')
+require_relative "spec_helper"
 
 describe "Database schema parser" do
-  before do
-    @iom = DB.identifier_output_method
-    @iim = DB.identifier_input_method
-    @qi = DB.quote_identifiers?
-  end
   after do
-    DB.identifier_output_method = @iom
-    DB.identifier_input_method = @iim
-    DB.quote_identifiers = @qi
     DB.drop_table?(:items)
   end
 
-  it "should handle a database with a identifier methods" do
-    DB.identifier_output_method = :reverse
-    DB.identifier_input_method = :reverse
-    DB.quote_identifiers = true
-    DB.create_table!(:items){Integer :number}
-    begin
-      DB.schema(:items, :reload=>true).must_be_kind_of(Array)
-      DB.schema(:items, :reload=>true).first.first.must_equal :number
-    ensure 
-      DB.drop_table(:items)
+  describe "with identifier mangling" do
+    before do
+      @iom = DB.identifier_output_method
+      @iim = DB.identifier_input_method
+      @qi = DB.quote_identifiers?
+    end
+    after do
+      DB.identifier_output_method = @iom
+      DB.identifier_input_method = @iim
+      DB.quote_identifiers = @qi
     end
-  end
 
-  it "should handle a dataset with identifier methods different than the database's" do
-    DB.identifier_output_method = :reverse
-    DB.identifier_input_method = :reverse
-    DB.quote_identifiers = true
-    DB.create_table!(:items){Integer :number}
-    DB.identifier_output_method = @iom
-    DB.identifier_input_method = @iim
-    ds = DB[:items]
-    ds.identifier_output_method = :reverse
-    ds.identifier_input_method = :reverse
-    begin
-      DB.schema(ds, :reload=>true).must_be_kind_of(Array)
-      DB.schema(ds, :reload=>true).first.first.must_equal :number
-    ensure 
+    it "should handle a database with a identifier methods" do
       DB.identifier_output_method = :reverse
       DB.identifier_input_method = :reverse
-      DB.drop_table(:items)
+      DB.quote_identifiers = true
+      DB.create_table!(:items){Integer :number}
+      begin
+        DB.schema(:items, :reload=>true).must_be_kind_of(Array)
+        DB.schema(:items, :reload=>true).first.first.must_equal :number
+      ensure 
+      end
     end
-  end
+
+    it "should handle a dataset with identifier methods different than the database's" do
+      DB.identifier_output_method = :reverse
+      DB.identifier_input_method = :reverse
+      DB.quote_identifiers = true
+      DB.create_table!(:items){Integer :number}
+      DB.identifier_output_method = @iom
+      DB.identifier_input_method = @iim
+      ds = DB[:items].
+        with_identifier_output_method(:reverse).
+        with_identifier_input_method(:reverse)
+      begin
+        DB.schema(ds, :reload=>true).must_be_kind_of(Array)
+        DB.schema(ds, :reload=>true).first.first.must_equal :number
+      ensure 
+        DB.identifier_output_method = :reverse
+        DB.identifier_input_method = :reverse
+        DB.drop_table(:items)
+      end
+    end
+  end if IDENTIFIER_MANGLING && !DB.frozen?
 
   it "should not issue an sql query if the schema has been loaded unless :reload is true" do
     DB.create_table!(:items){Integer :number}
@@ -106,7 +110,7 @@ describe "Database schema parser" do
 
   it "should parse defaults from the schema properly" do
     DB.create_table!(:items){Integer :number}
-    DB.schema(:items).first.last[:ruby_default].must_equal nil
+    DB.schema(:items).first.last[:ruby_default].must_be_nil
     DB.create_table!(:items){Integer :number, :default=>0}
     DB.schema(:items).first.last[:ruby_default].must_equal 0
     DB.create_table!(:items){String :a, :default=>"blah"}
@@ -115,7 +119,7 @@ describe "Database schema parser" do
 
   it "should make :default nil for a NULL default" do
     DB.create_table!(:items){Integer :number}
-    DB.schema(:items).first.last[:default].must_equal nil
+    DB.schema(:items).first.last[:default].must_be_nil
     DB.create_table!(:items){Integer :number, :default=>0}
     DB.schema(:items).first.last[:default].wont_equal nil
   end
@@ -159,6 +163,18 @@ describe "Database schema parser" do
     DB.schema(:items).first.last[:type].must_equal :boolean
   end
 
+  it "should round trip database types from the schema properly" do
+    DB.create_table!(:items){String :number, :size=>50}
+    db_type = DB.schema(:items).first.last[:db_type]
+    DB.create_table!(:items){column :number, db_type}
+    DB.schema(:items).first.last[:db_type].must_equal db_type
+
+    DB.create_table!(:items){Numeric :number, :size=>[11,3]}
+    db_type = DB.schema(:items).first.last[:db_type]
+    DB.create_table!(:items){column :number, db_type}
+    DB.schema(:items).first.last[:db_type].must_equal db_type
+  end
+
   it "should parse maximum length for string columns" do
     DB.create_table!(:items){String :a, :size=>4}
     DB.schema(:items).first.last[:max_length].must_equal 4
@@ -199,7 +215,7 @@ describe "Database index parsing" do
     DB.indexes(:items).must_equal({})
   end
 
-  cspecify "should not include partial indexes", :sqlite do
+  it "should not include partial indexes" do
     DB.create_table!(:items){Integer :n; Integer :a; index :n, :where=>proc{n > 10}}
     DB.indexes(:items).must_equal({})
   end if DB.supports_partial_indexes?
@@ -259,6 +275,12 @@ describe "Database foreign key parsing" do
     @db.create_table!(:b, :engine=>:InnoDB){Integer :e, :null=>false; Integer :f, :null=>false; Integer :g, :null=>false; foreign_key [:e, :f], :a; foreign_key [:g, :f], :a, :key=>[:d, :c]}
     @pr[:b, [[:e, :f], :a, [:pk, :b, :c]], [[:g, :f], :a, [:d, :c]]]
   end
+
+  it "should handle self-referential composite foreign and primary keys" do
+    @db.create_table!(:a, :engine=>:InnoDB){Integer :b, :null=>false; Integer :c, :null=>false; Integer :d, :null=>false; primary_key [:b, :c]; unique [:d, :b]}
+    @db.alter_table(:a){add_foreign_key [:b, :d], :a; add_foreign_key [:d, :c], :a; add_foreign_key [:c, :b], :a, :key=>[:d, :b]}
+    @pr[:a, [[:b, :d], :a, [:pk, :b, :c]], [[:c, :b], :a, [:d, :b]], [[:d, :c], :a, [:pk, :b, :c]]]
+  end
 end if DB.supports_foreign_key_parsing?
 
 describe "Database schema modifiers" do
@@ -336,7 +358,7 @@ describe "Database schema modifiers" do
       @ds.select_order_map(:number).must_equal [1, 2, 2, 3, 4]
     end if DB.supports_views_with_local_check_option?
 
-    cspecify "should create views with explicit columns correctly", :sqlite do
+    cspecify "should create views with explicit columns correctly", [proc{|db| db.sqlite_version < 30900}, :sqlite] do
       @db.create_view(:items_view, @ds.where(:number=>1), :columns=>[:n])
       @db[:items_view].map(:n).must_equal [1]
     end
@@ -365,6 +387,26 @@ describe "Database schema modifiers" do
     @db.table_exists?(:items).must_equal false
   end if DB.supports_transactional_ddl?
   
+  it "should handle errors creating indexes when ignoring index errors" do
+    @db.drop_table?(:items)
+    @db.transaction do
+      @db.create_table(:items, :ignore_index_errors=>true) do
+        Integer :n1
+        Integer :n2
+        index :n1, :name=>'items_n1'
+        index :foo, :name=>'items_f'
+        index :n2, :name=>'items_n2'
+        index :bar, :name=>'items_g'
+      end
+    end
+    @db.table_exists?(:items).must_equal true
+    indexes = @db.indexes(:items).keys
+    indexes.must_include :items_n1
+    indexes.must_include :items_n2
+    indexes.wont_include :items_f
+    indexes.wont_include :items_g
+  end if DB.supports_transactional_ddl? && DB.database_type != :mssql
+  
   describe "join tables" do
     after do
       @db.drop_join_table(:cat_id=>:cats, :dog_id=>:dogs) if @db.table_exists?(:cats_dogs)
@@ -497,6 +539,27 @@ describe "Database schema modifiers" do
     proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
   end
 
+  it "should not allow NULLs when adding a primary key column" do
+    @db.create_table!(:items){String :foo}
+    @db.alter_table(:items){add_column :id, String, :primary_key=>true, :default=>'a'}
+    proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
+  it "should not allow NULLs when creating table with primary key constraint" do
+    @db.create_table!(:items){String :id1; String :id2; primary_key [:id1, :id2]}
+    proc{@ds.insert(:id1=>nil, :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>nil, :id2=>'1')}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>'1', :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
+  it "should not allow NULLs when adding a primary key constraint" do
+    @db.create_table!(:items){String :id1; String :id2}
+    @db.alter_table(:items){add_primary_key [:id1, :id2]}
+    proc{@ds.insert(:id1=>nil, :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>nil, :id2=>'1')}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+    proc{@ds.insert(:id1=>'1', :id2=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
+  end
+
   it "should rename columns correctly" do
     @db.create_table!(:items){Integer :id}
     @ds.insert(:id=>10)
@@ -547,7 +610,7 @@ describe "Database schema modifiers" do
     @ds.all.must_equal [{:id2=>10}]
   end
 
-  cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2], [:db2] do
+  cspecify "should set column NULL/NOT NULL correctly", [:jdbc, :db2] do
     @db.create_table!(:items, :engine=>:InnoDB){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_allow_null :id, false}
@@ -569,17 +632,17 @@ describe "Database schema modifiers" do
     @ds.all.must_equal [{:id=>10}, {:id=>20}]
   end
 
-  cspecify "should set column types correctly", [:jdbc, :db2], [:db2], :oracle do
+  cspecify "should set column types correctly", [:jdbc, :db2], :oracle do
     @db.create_table!(:items){Integer :id}
     @ds.insert(:id=>10)
     @db.alter_table(:items){set_column_type :id, String}
     @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
     @ds.columns!.must_equal [:id]
     @ds.insert(:id=>'20')
-    @ds.all.must_equal [{:id=>"10"}, {:id=>"20"}]
+    @ds.order(:id).all.must_equal [{:id=>"10"}, {:id=>"20"}]
   end
 
-  cspecify "should set column types without modifying NULL/NOT NULL", [:jdbc, :db2], [:db2], :oracle, :derby do
+  cspecify "should set column types without modifying NULL/NOT NULL", [:jdbc, :db2], :derby do
     @db.create_table!(:items){Integer :id, :null=>false, :default=>2}
     proc{@ds.insert(:id=>nil)}.must_raise(Sequel::NotNullConstraintViolation, Sequel::ConstraintViolation, Sequel::DatabaseError)
     @db.alter_table(:items){set_column_type :id, String}
@@ -592,7 +655,7 @@ describe "Database schema modifiers" do
     @ds.map(:id).must_equal [nil, nil]
   end
 
-  cspecify "should set column types without modifying defaults", [:jdbc, :db2], [:db2], :oracle, :derby do
+  cspecify "should set column types without modifying defaults", [:jdbc, :db2], :oracle, :derby do
     @db.create_table!(:items){Integer :id, :default=>0}
     @ds.insert
     @ds.map(:id).must_equal [0]
@@ -697,7 +760,7 @@ describe "Database schema modifiers" do
     @db.schema(:items, :reload=>true).map{|x| x.first}.must_equal [:id]
   end
 
-  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2], [:db2] do
+  cspecify "should work correctly with many operations in a single alter_table call", [:jdbc, :db2] do
     @db.create_table!(:items) do
       primary_key :id
       String :name2
@@ -744,7 +807,7 @@ describe "Database schema modifiers" do
   end if DB.supports_deferrable_constraints?
 end
 
-describe "Database#tables" do
+describe "Database#tables and #views" do
   before do
     class ::String
       @@xxxxx = 0
@@ -753,65 +816,52 @@ describe "Database#tables" do
       end
     end
     @db = DB
+    @db.drop_view(:sequel_test_view) rescue nil
+    @db.drop_table?(:sequel_test_table)
     @db.create_table(:sequel_test_table){Integer :a}
     @db.create_view :sequel_test_view, @db[:sequel_test_table]
-    @iom = @db.identifier_output_method
-    @iim = @db.identifier_input_method
   end
   after do
-    @db.identifier_output_method = @iom
-    @db.identifier_input_method = @iim
     @db.drop_view :sequel_test_view
     @db.drop_table :sequel_test_table
   end
 
-  it "should return an array of symbols" do
+  it "#tables should return an array of symbols" do
     ts = @db.tables
     ts.must_be_kind_of(Array)
     ts.each{|t| t.must_be_kind_of(Symbol)}
     ts.must_include(:sequel_test_table)
     ts.wont_include(:sequel_test_view)
-  end
+  end if DB.supports_table_listing?
 
-  it "should respect the database's identifier_output_method" do
-    @db.identifier_output_method = :xxxxx
-    @db.identifier_input_method = :xxxxx
-    @db.tables.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
-  end
-end if DB.supports_table_listing?
-
-describe "Database#views" do
-  before do
-    class ::String
-      @@xxxxx = 0
-      def xxxxx
-        "xxxxx#{@@xxxxx += 1}"
-      end
-    end
-    @db = DB
-    @db.create_table(:sequel_test_table){Integer :a}
-    @db.create_view :sequel_test_view, @db[:sequel_test_table]
-    @iom = @db.identifier_output_method
-    @iim = @db.identifier_input_method
-  end
-  after do
-    @db.identifier_output_method = @iom
-    @db.identifier_input_method = @iim
-    @db.drop_view :sequel_test_view
-    @db.drop_table :sequel_test_table
-  end
-
-  it "should return an array of symbols" do
+  it "#views should return an array of symbols" do
     ts = @db.views
     ts.must_be_kind_of(Array)
     ts.each{|t| t.must_be_kind_of(Symbol)}
     ts.wont_include(:sequel_test_table)
     ts.must_include(:sequel_test_view)
-  end
+  end if DB.supports_view_listing?
 
-  it "should respect the database's identifier_output_method" do
-    @db.identifier_output_method = :xxxxx
-    @db.identifier_input_method = :xxxxx
-    @db.views.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
-  end
-end if DB.supports_view_listing?
+  describe "with identifier mangling" do
+    before do
+      @iom = @db.identifier_output_method
+      @iim = @db.identifier_input_method
+    end
+    after do
+      @db.identifier_output_method = @iom
+      @db.identifier_input_method = @iim
+    end
+
+    it "#tables should respect the database's identifier_output_method" do
+      @db.identifier_output_method = :xxxxx
+      @db.identifier_input_method = :xxxxx
+      @db.tables.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
+    end if DB.supports_table_listing?
+
+    it "#views should respect the database's identifier_output_method" do
+      @db.identifier_output_method = :xxxxx
+      @db.identifier_input_method = :xxxxx
+      @db.views.each{|t| t.to_s.must_match(/\Ax{5}\d+\z/)}
+    end if DB.supports_view_listing?
+  end if IDENTIFIER_MANGLING && !DB.frozen?
+end
